@@ -23,11 +23,11 @@ __all__ = (
 
     '_Field', '_Text', '_Password', '_Number', '_NaturalNumber', '_Email',
     '_URL', '_Date', '_Color', '_File', '_Boolean',
-    '_Select', '_SelectMulti', '_Collection',
+    '_Select', '_Collection',
 
     'Field', 'Text', 'Password', 'Number', 'NaturalNumber', 'Integer', 'Email',
     'URL', 'Date', 'Color', 'File', 'Boolean',
-    'Select', 'SelectMulti', 'Collection',
+    'Select', 'Collection',
 )
 
 
@@ -41,9 +41,11 @@ class ValidationError(object):
         return self.message
 
 
+
 #- Real fields
 #------------------------------------------------------------------------------#
 
+
 class _Field(object):
     """The real form field class.
 
@@ -51,35 +53,65 @@ class _Field(object):
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
-    Any other named parameter will be stored in `self.extra`.
-    """
+    :param default:
+        Default value.
 
-    _value = None
-    _python_value = None
-    original_value = None
-    error = None
-    name = 'unnamed'
+    :param prepare:
+        An optional function that preprocess the value before loading.
+
+    :param clean:
+        An optional function that takes the value and return a 'cleaned'
+        version of it. If the value raise an exception, `None` must be
+        returned instead.
+
+    :param locale:
+        Default locale
+
+    :param tz:
+        Default timezone
+
+    """
+    name = 'field'
     locale = 'en'
     tz = utc
 
+    value = None
+    obj_value = None
+
+    _default_validator = None
+    
+    error = None
     has_changed = False
+
+    # Do not show the current value of the field in the HTML
     hide_value = False
+
     # Indicates that the user hasn't sent data for this field
     empty = True
 
-    def __init__(self, validate=None, default=u'', **kwargs):
+    def __init__(self, validate=None, default=None, **kwargs):
         validate = validate or []
+        defval = self._default_validator
+        validate = list(validate)
+        if defval and not self._validator_in(defval, validate):
+            validate.append(defval())
         self.validators = [val() if inspect.isclass(val) else val
             for val in validate]
+
         self.optional = not self._validator_in(v.Required, validate)
         self.default = default
+
         # Extensibility FTW
+        self.custom_prepare = kwargs.pop('prepare', None)
+        self.custom_clean = kwargs.pop('clean', None)
+        self.locale = kwargs.pop('locale', self.locale)
+        self.tz = kwargs.pop('tz', self.tz)
         self.extra = kwargs
 
     def reset(self):
-        self._value = None
-        self.python_value = None
-        self.original_value = None
+        self.value = None
+        self.obj_value = None
+        self.empty = True
 
     def _validator_in(self, validator, validators):
         for v in validators:
@@ -87,90 +119,62 @@ class _Field(object):
                 return True
         return False
 
-    def _init(self, data=None, original_value=None, files=None,
-            locale='en', tz=utc):
+    def load_value(self, data=None, files=None, obj_value=None, locale=None, tz=None):
+        self.reset()
         self.empty = not bool(data or files)
         self.value = data or files
-        if not self._value:
-            self.python_value = original_value
-        self.original_value = original_value
-        self.locale = locale
-        self.tz = tz
+        self.obj_value = obj_value
+        self.locale = locale or self.locale
+        self.tz = tz or self.tz
 
     def get_value(self):
-        if self.hide_value:
-            return u''
-        if self._value:
-            return self._value
-        return self.to_html(locale=self.locale, tz=self.tz)
-
-    def set_value(self, value):
+        value = self.value or self.obj_value
         if isinstance(value, list) and value:
-            value = value[0] or u''
-        elif value is None:
-            value = u''
-        if isinstance(value, basestring):
-            value = value.strip()
-        self._value = value
-        self._python_value = None
+            value = value[0]
+        return value
 
-    def _get_value(self):
-        return self.get_value()
-
-    def _set_value(self, value):
-        self.set_value(value)
-
-    value = property(_get_value, _set_value)
-
-    def get_python_value(self, locale=None, tz=None):
-        if self._python_value:
-            return self._python_value
-        return self.to_python(locale, tz)
-
-    def set_python_value(self, python_value):
-        self._python_value = python_value
-
-    def _get_python_value(self):
-        return self.get_python_value()
-
-    def _set_python_value(self, value):
-        self.set_python_value(value)
-
-    python_value = property(_get_python_value, _set_python_value)
+    def prepare(self, value, locale=None, tz=None):
+        return to_unicode(value or u'')
 
     def to_html(self, locale=None, tz=None):
-        return to_unicode(self._python_value or u'')
+        if self.hide_value:
+            return u''
+        value = self.get_value()
+        if self.custom_prepare:
+            value = self.custom_prepare(value, locale, tz)
+        value = self.prepare(value, locale, tz)
+        return value.strip()
+
+    def clean(self, value, locale=None, tz=None):
+        return value
 
     def to_python(self, locale=None, tz=None):
-        if not self._value:
-            return None
-        return self._value
+        value = self.get_value()
+        if self.custom_clean:
+            value = self.custom_clean(value, locale, tz)
+        value = self.clean(value, locale, tz)
+        return value
 
-    def clean_value(self, python_value):
-        return python_value
-
-    def validate(self, form=None, cleaned_data=None):
+    def validate(self, form=None, cleaned_data=None, locale=None, tz=None):
         """Validates the current value of a field.
         """
         if cleaned_data is None:
             self.error = None
-            python_value = self.python_value
-            python_value = self.clean_value(python_value)
+            python_value = self.to_python(locale, tz)
 
             if isinstance(python_value, ValidationError):
                 self.error = python_value
                 return None
 
-            hs = (python_value or '') != (self.original_value or '')
-            self.has_changed = hs
+            self.has_changed = (python_value != self.obj_value)
             # Do not validate optional fields
             if (python_value is None) and self.optional:
                 return self.default or None
 
-            return self._validate_value(python_value, form)
-        self._validate_form(cleaned_data, form)
+            return self._validate_value(form, python_value)
+        self._validate_form(form, cleaned_data)
 
-    def _validate_value(self, python_value, form):
+    def _validate_value(self, form, python_value):
         for val in self.validators:
             if isinstance(val, v.FormValidator):
                 continue
@@ -179,7 +183,7 @@ class _Field(object):
                 return self.default
         return python_value
 
-    def _validate_form(self, cleaned_data, form):
+    def _validate_form(self, form, cleaned_data):
         for val in self.validators:
             if not isinstance(val, v.FormValidator):
                 continue
@@ -256,47 +260,54 @@ class _Field(object):
 class _Text(_Field):
     """A text field.
 
-    :param clean:
-        An optional function that takes the value and return a 'cleaned'
-        version of it. If the value raise an exception, `None` will be
-        returned instead.
     :param validate:
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
-    Any other named parameter will be stored in `self.extra`.
+    :param default:
+        Default value.
+
+    :param prepare:
+        An optional function that preprocess the value before loading.
+
+    :param clean:
+        An optional function that takes the value and return a 'cleaned'
+        version of it. If the value raise an exception, `None` must be
+        returned instead.
+
+    :param locale:
+        Default locale
+
+    :param tz:
+        Default timezone
+
     """
     _type = 'text'
     _default_validator = None
 
-    def __init__(self, validate=None, **kwargs):
-        validate = validate or []
-        defval = self._default_validator
-        validate = list(validate)
-        if defval and not self._validator_in(defval, validate):
-            validate.append(defval())
-
-        super(_Text, self).__init__(validate=validate, **kwargs)
+    def __init__(self, **kwargs):
+        super(_Text, self).__init__(**kwargs)
 
     def __call__(self, **kwargs):
         return self.as_input(**kwargs)
 
-    def as_input(self, **kwargs):
+    def as_input(self, locale=None, tz=None, **kwargs):
         kwargs['type'] = kwargs.get('type', self._type)
         kwargs['name'] = self.name
-        kwargs['value'] = self.value
+        kwargs['value'] = self.to_html(locale, tz)
         if not self.optional:
             kwargs['required'] = True
         html_attrs = self._get_html_attrs(kwargs)
         html = u'<input %s>' % html_attrs
         return Markup(html)
 
-    def as_textarea(self, **kwargs):
+    def as_textarea(self, locale=None, tz=None, **kwargs):
         kwargs['name'] = self.name
         if not self.optional:
             kwargs['required'] = True
         html_attrs = self._get_html_attrs(kwargs)
-        html = u'<textarea %s>%s</textarea>' % (html_attrs, self.value)
+        value = self.to_html(locale, tz)
+        html = u'<textarea %s>%s</textarea>' % (html_attrs, value)
         return Markup(html)
 
 
@@ -306,18 +317,35 @@ class _Password(_Text):
     :param hide_value:
         If `True` this field will not reproduce the value on a form
         submit by default. This is the default for security purposes.
+
     :param validate:
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
-    Any other named parameter will be stored in `self.extra`.
+    :param default:
+        Default value.
+
+    :param prepare:
+        An optional function that preprocess the value before loading.
+
+    :param clean:
+        An optional function that takes the value and return a 'cleaned'
+        version of it. If the value raise an exception, `None` must be
+        returned instead.
+
+    :param locale:
+        Default locale
+
+    :param tz:
+        Default timezone
+    
     """
     _type = 'password'
+    hide_value = True
 
-    def __init__(self, hide_value=True, validate=None, **kwargs):
-        validate = validate or []
+    def __init__(self, hide_value=True, **kwargs):
         self.hide_value = hide_value
-        super(_Password, self).__init__(validate=validate, **kwargs)
+        super(_Password, self).__init__(**kwargs)
 
 
 class _Number(_Text):
@@ -327,15 +355,30 @@ class _Number(_Text):
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
-    Any other named parameter will be stored in `self.extra`.
+    :param default:
+        Default value.
+
+    :param prepare:
+        An optional function that preprocess the value before loading.
+
+    :param clean:
+        An optional function that takes the value and return a 'cleaned'
+        version of it. If the value raise an exception, `None` must be
+        returned instead.
+
+    :param locale:
+        Default locale
+
+    :param tz:
+        Default timezone
+    
     """
     _type = 'number'
     _default_validator = v.IsNumber
-    default = None
 
-    def to_python(self, locale=None, tz=None):
+    def clean(self, value, locale=None, tz=None):
         try:
-            return float(self._value)
+            return float(value)
         except Exception:
             return None
 
@@ -347,15 +390,30 @@ class _NaturalNumber(_Text):
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
-    Any other named parameter will be stored in `self.extra`.
+    :param default:
+        Default value.
+
+    :param prepare:
+        An optional function that preprocess the value before loading.
+
+    :param clean:
+        An optional function that takes the value and return a 'cleaned'
+        version of it. If the value raise an exception, `None` must be
+        returned instead.
+
+    :param locale:
+        Default locale
+
+    :param tz:
+        Default timezone
+    
     """
     _type = 'number'
     _default_validator = v.IsNaturalNumber
-    default = None
 
-    def to_python(self, locale=None, tz=None):
+    def clean(self, value, locale=None, tz=None):
         try:
-            return int(str(self._value), 10)
+            return int(str(value), 10)
         except Exception:
             return None
 
@@ -367,7 +425,23 @@ class _Email(_Text):
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
-    Any other named parameter will be stored in `self.extra`.
+    :param default:
+        Default value.
+
+    :param prepare:
+        An optional function that preprocess the value before loading.
+
+    :param clean:
+        An optional function that takes the value and return a 'cleaned'
+        version of it. If the value raise an exception, `None` must be
+        returned instead.
+
+    :param locale:
+        Default locale
+
+    :param tz:
+        Default timezone
+
     """
     _type = 'email'
     _default_validator = v.ValidEmail
@@ -380,7 +454,23 @@ class _URL(_Text):
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
-    Any other named parameter will be stored in `self.extra`.
+    :param default:
+        Default value.
+
+    :param prepare:
+        An optional function that preprocess the value before loading.
+
+    :param clean:
+        An optional function that takes the value and return a 'cleaned'
+        version of it. If the value raise an exception, `None` must be
+        returned instead.
+
+    :param locale:
+        Default locale
+
+    :param tz:
+        Default timezone
+
     """
     _type = 'url'
     _default_validator = v.ValidURL
@@ -393,31 +483,47 @@ class _Date(_Text):
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
-    Any other named parameter will be stored in `self.extra`.
+    :param default:
+        Default value.
+
+    :param prepare:
+        An optional function that preprocess the value before loading.
+
+    :param clean:
+        An optional function that takes the value and return a 'cleaned'
+        version of it. If the value raise an exception, `None` must be
+        returned instead.
+
+    :param locale:
+        Default locale
+
+    :param tz:
+        Default timezone
+
     """
     _type = 'datetime'
     _default_validator = v.IsDate
     format = 'short'
 
-    def __init__(self, format=None, *args, **kwargs):
+    def __init__(self, format=None, **kwargs):
         self.format = format
-        return super(_Date, self).__init__(*args, **kwargs)
+        return super(_Date, self).__init__(**kwargs)
 
-    def set_value(self, value):
+    def prepare(self, value, locale=None, tz=None):
         if isinstance(value, list) and value:
             value = value[0] or u''
         if isinstance(value, date) and not isinstance(value, datetime):
             now = datetime.utcnow()
             value = datetime(value.year, value.month, value.day,
                 now.hour, now.minute, now.second)
-        return super(_Date, self).set_value(value)
+        return value
 
     def to_html(self, locale=None, tz=None):
         locale = locale or self.locale or 'en'
         tz = tz or self.tz
         if isinstance(tz, basestring):
             tz = timezone(tz)
-        value = self.python_value
+        value = self.value
         if isinstance(value, date) and not isinstance(value, datetime):
             now = datetime.utcnow()
             value = datetime(value.year, value.month, value.day,
@@ -428,19 +534,18 @@ class _Date(_Text):
             # raise
             return u''
 
-    def to_python(self, locale=None, tz=None):
-        if not self._value:
+    def clean(self, value, locale=None, tz=None):
+        if not value:
             return None
         locale = locale or self.locale or 'en'
         tz = tz or self.tz
         if isinstance(tz, basestring):
             tz = timezone(tz)
-
         try:
             try:
-                dt = parse_datetime(self._value, locale=locale)
+                dt = parse_datetime(value, locale=locale)
             except NotImplementedError:
-                dt = parse_date(self._value, locale=locale)
+                dt = parse_date(value, locale=locale)
             if isinstance(dt, date) and not isinstance(dt, datetime):
                 now = datetime.utcnow()
                 dt = datetime(dt.year, dt.month, dt.day,
@@ -460,7 +565,23 @@ class _Color(_Text):
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
-    Any other named parameter will be stored in `self.extra`.
+    :param default:
+        Default value.
+
+    :param prepare:
+        An optional function that preprocess the value before loading.
+
+    :param clean:
+        An optional function that takes the value and return a 'cleaned'
+        version of it. If the value raise an exception, `None` must be
+        returned instead.
+
+    :param locale:
+        Default locale
+
+    :param tz:
+        Default timezone
+    
     """
     _type = 'color'
     _default_validator = v.IsColor
@@ -471,10 +592,10 @@ class _Color(_Text):
         r'(?:,(?P<a>[0-9]+))?\)',
         re.IGNORECASE)
 
-    def to_python(self, locale=None, tz=None):
-        if not self._value:
+    def clean(self, value, locale=None, tz=None):
+        if not value:
             return None
-        m = self._re_colors.match(self._value.replace(' ', ''))
+        m = self._re_colors.match(value.replace(' ', ''))
         if not m:
             return None
         md = m.groupdict()
@@ -518,29 +639,45 @@ class _File(_Field):
     :param upload:
         Optional function to be call for doing the actual file upload. It must
         return a python value ready for validation.
+
     :param validate:
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
-    Any other named parameter will be stored in `self.extra`.
+    :param default:
+        Default value.
+
+    :param prepare:
+        An optional function that preprocess the value before loading.
+
+    :param clean:
+        An optional function that takes the value and return a 'cleaned'
+        version of it. If the value raise an exception, `None` must be
+        returned instead.
+
+    :param locale:
+        Default locale
+
+    :param tz:
+        Default timezone
+    
     """
     hide_value = True
 
-    def __init__(self, upload=None, validate=None, **kwargs):
-        validate = validate or []
+    def __init__(self, upload=None, **kwargs):
         self.upload = upload
-        super(_File, self).__init__(validate=validate, **kwargs)
-
-    def to_html(self, locale=None, tz=None):
-        return self.python_value
+        super(_File, self).__init__(**kwargs)
 
     def to_python(self, locale=None, tz=None):
-        if not self._value:
-            return self.original_value
+        value = self.value
+        if isinstance(value, list) and value:
+            value = value[0]
+        if not value:
+            return self.obj_value
         if not self.upload:
-            return self._value
+            return value
         try:
-            return self.upload(self._value)
+            return self.upload(value)
         except Exception, e:
             return ValidationError('invalid_file', str(e))
 
@@ -564,20 +701,36 @@ class _Boolean(_Field):
 
     :param falsy:
         A list of raw values considered `False`.
+
     :param validate:
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
-    Any other named parameter will be stored in `self.extra`.
+    :param default:
+        Default value.
+
+    :param prepare:
+        An optional function that preprocess the value before loading.
+
+    :param clean:
+        An optional function that takes the value and return a 'cleaned'
+        version of it. If the value raise an exception, `None` must be
+        returned instead.
+
+    :param locale:
+        Default locale
+
+    :param tz:
+        Default timezone
+
     """
 
-    def __init__(self, falsy=FALSY_VALUES, validate=None, **kwargs):
-        validate = validate or []
+    def __init__(self, falsy=FALSY_VALUES, **kwargs):
         self.falsy = falsy
-        super(_Boolean, self).__init__(validate=validate, **kwargs)
+        super(_Boolean, self).__init__(**kwargs)
 
     def to_python(self, locale=None, tz=None):
-        value = self._value
+        value = self.value
         if not value or (value in self.falsy):
             return False
         return True
@@ -597,141 +750,113 @@ class _Boolean(_Field):
         return Markup(html)
 
 
-class _Select(_Field):
+
+#- Multivalue
+#------------------------------------------------------------------------------#
+
+
+class MultiValue(_Field):
+    """
+    :param filters:
+        List of validators. If a value do not pass one of these it'll be
+        filtered out from the final result.
+    """
+    def __init__(self, filters=None, **kwargs):
+        filters = filters or []
+        self.filters = [f() if inspect.isclass(f) else f for f in filters]
+        super(MultiValue, self).__init__(**kwargs)
+
+    def get_value(self):
+        values = self.value or self.obj_value
+        if not values:
+            return []
+        if isinstance(values, list):
+            return values
+        return [values]
+
+    def prepare(self, values, locale=None, tz=None):
+        if self.custom_prepare:
+            values_ = []
+            for val in values:
+                try:
+                    val = self.custom_prepare(val, locale, tz)        
+                except (TypeError, ValueError):
+                    try:
+                        val = self.custom_prepare(val)        
+                    except (TypeError, ValueError):
+                        continue
+                if val:
+                    values_.append(val)
+            values = values_
+        return values
+
+    def clean(self, values, locale=None, tz=None):
+        for f in self.filters:
+            values = filter(f, values)
+
+        if self.custom_clean:
+            values_ = []
+            for val in values:
+                try:
+                    val = self.custom_clean(val, locale, tz)        
+                except (TypeError, ValueError):
+                    try:
+                        val = self.custom_clean(val)        
+                    except (TypeError, ValueError):
+                        continue
+                if val:
+                    values_.append(val)
+            values = values_
+        return filter(lambda v: bool(v), values)
+
+    def to_python(self, locale=None, tz=None):
+        values = self.get_value()
+        values = self.clean(values, locale, tz)
+        return values
+
+
+class _Select(MultiValue):
     """A field with a fixed list of options for the values
 
     :param items:
         Either: 
         - An list of tuples with the format `(value, label)`; or
         - A function that return a list of items in that format.
-    :param clean:
-        An optional function that takes the value and return a 'cleaned'
-        version of it. If the value raise an exception, `None` will be
-        returned instead.
+
+    :param multiple:
+        Whether or not more than one value can be selected
+
+    :param filters:
+        List of validators. If a value do not pass one of these it'll be
+        filtered out from the final result.
+
     :param validate:
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
-    Any other named parameter will be stored in `self.extra`.
-    """
+    :param default:
+        Default value.
 
-    def __init__(self, items, clean=None, validate=None, **kwargs):
-        validate = validate or []
-        self.items = items
-        self.clean = clean
-        super(_Select, self).__init__(validate=validate, **kwargs)
+    :param prepare:
+        An optional function that preprocess the value before loading.
 
-    def get_items(self):
-        return self.items() if callable(self.items) else self.items
-
-    def to_python(self, locale=None, tz=None):
-        value = self._value
-        if self.clean:
-            try:
-                value = self.clean(value)
-            except Exception:
-                return None
-        return value
-
-    def __call__(self, **kwargs):
-        items = self.get_items()
-        if len(items) > 5:
-            return self.as_select(_items=items, **kwargs)    
-        return self.as_radiobuttons(_items=items, **kwargs)
-
-    def as_select(self, _items=None, no_value=None, **kwargs):
-        """Render the field as `<select>` element.
-        
-        :param **kwargs:
-            Named paremeters used to generate the HTML attributes of each item.
-            It follows the same rules as `Field._get_html_attrs`
-        
-        """
-        kwargs['name'] = self.name
-        if not self.optional:
-            kwargs['required'] = True
-        html_attrs = self._get_html_attrs(kwargs)
-        html = [u'<select %s>' % html_attrs]
-
-        if _items is None:
-            _items = self.get_items()
-        curr_value = self.value
-        if no_value:
-            _items.insert(('', no_value))
-        
-        for value, label in _items:
-            item_attrs = {'value': value}
-            if str(value) == curr_value:
-                item_attrs['selected'] = True
-            html_attrs = self._get_html_attrs(item_attrs)
-            html.append(u'<option %s>%s</option>' % (html_attrs, label))
-        html.append(u'</select>')
-        return Markup('\n'.join(html))
-
-    def as_radiobuttons(self, _items=None, 
-            tmpl=u'<label><input %(attrs)s> %(label)s</label>', **kwargs):
-        """Render the field as a series of radio buttons, using the `tmpl`
-        parameter as the template.
-        
-        :param tmpl:
-            HTML template to use for rendering each item.
-        :param **kwargs:
-            Named paremeters used to generate the HTML attributes of each item.
-            It follows the same rules as `Field._get_html_attrs`
-        
-        """
-        if _items is None:
-            _items = self.get_items()
-        kwargs['type'] = 'radio'
-        kwargs['name'] = self.name
-        html = []
-        curr_value = self.value
-        for value, label in _items:
-            item_attrs = kwargs.copy()
-            item_attrs['value'] = value
-            if str(value) == curr_value:
-                item_attrs['checked'] = True
-            html_attrs = self._get_html_attrs(item_attrs)
-            html.append(tmpl % {'attrs': html_attrs, 'label': label})
-        return Markup('\n'.join(html))
-
-
-class _SelectMulti(_Field):
-    """A field with a fixed list of options for the values.
-    Similar to `Select`, except this one can take (and validate)
-    multiple choices.
-    
-    :param items:
-        Either: 
-        - An list of tuples with the format (value, label); or
-        - A function that return a list of items in that format.
     :param clean:
-        An optional function that takes a value and return a 'cleaned' version
-        of it. If a value raise an exception it'll be filtered out from the
-        final result.
-    :param validate:
-        An list of validators. This will evaluate each of the selected values
-        when the method `validate` is called.
+        An optional function that takes the value and return a 'cleaned'
+        version of it. If the value raise an exception, `None` must be
+        returned instead.
+
+    :param locale:
+        Default locale
+
+    :param tz:
+        Default timezone
 
-    Any other named parameter will be stored in `self.extra`.
     """
 
-    def __init__(self, items, clean=None, validate=None, **kwargs):
-        validate = validate or []
+    def __init__(self, items, multiple=False, **kwargs):
         self.items = items
-        self.clean = clean
-        super(_SelectMulti, self).__init__(validate=validate, **kwargs)
-
-    def get_value(self):
-        return self._value if self._value and not self.hide_value else []
-
-    def set_value(self, value):
-        if not isinstance(value, list):
-            value = [value]
-        elif value is None:
-            value = []
-        self._value = value
-        self._python_value = None
+        self.multiple = multiple
+        super(_Select, self).__init__(**kwargs)
 
     def get_items(self):
         return self.items() if callable(self.items) else self.items
@@ -741,23 +866,19 @@ class _SelectMulti(_Field):
         for item in items:
             yield item
 
-    def to_python(self, locale=None, tz=None):
-        values = self._value
-        if self.clean:
-            values_ = []
-            for value in values:
-                try:
-                    values_.append(self.clean(value))
-                except Exception:
-                    pass
-            values = values_
-        return values
+    def to_html(self, locale=None, tz=None):
+        if self.hide_value:
+            return []
+        values = self.get_value()
+        return self.prepare(values, locale, tz)
 
     def __call__(self, **kwargs):
         items = self.get_items()
         if len(items) > 5:
-            return self.as_select(_items=items, **kwargs)    
-        return self.as_checkboxes(_items=items, **kwargs)
+            return self.as_select(_items=items, **kwargs)
+        if self.multiple:
+            return self.as_checkboxes(_items=items, **kwargs)
+        return self.as_radiobuttons(_items=items, **kwargs)
 
     def as_select(self, _items=None, **kwargs):
         """Render the field as `<select multiple>` element.
@@ -768,7 +889,7 @@ class _SelectMulti(_Field):
         
         """
         kwargs['name'] = self.name
-        kwargs['multiple'] = True
+        kwargs['multiple'] = self.multiple
         if not self.optional:
             kwargs['required'] = True
         html_attrs = self._get_html_attrs(kwargs)
@@ -776,10 +897,11 @@ class _SelectMulti(_Field):
 
         if _items is None:
             _items = self.get_items()
-        curr_values = self.value
+        curr_values = self.to_html()
+
         for value, label in _items:
             item_attrs = {'value': value}
-            if str(value) in curr_values:
+            if value in curr_values or str(value) in curr_values:
                 item_attrs['selected'] = True
             html_attrs = self._get_html_attrs(item_attrs)
             html.append(u'<option %s>%s</option>' % (html_attrs, label))
@@ -803,18 +925,50 @@ class _SelectMulti(_Field):
         kwargs['type'] = 'checkbox'
         kwargs['name'] = self.name
         html = []
-        curr_values = self.value
+        curr_values = self.to_html()
+        
         for value, label in _items:
             item_attrs = kwargs.copy()
             item_attrs['value'] = value
-            if str(value) in curr_values:
+            if value in curr_values or str(value) in curr_values:
+                item_attrs['checked'] = True
+            html_attrs = self._get_html_attrs(item_attrs)
+            html.append(tmpl % {'attrs': html_attrs, 'label': label})
+        return Markup('\n'.join(html))
+
+    def as_radiobuttons(self, _items=None, 
+            tmpl=u'<label><input %(attrs)s> %(label)s</label>', **kwargs):
+        """Render the field as a series of radio buttons, using the `tmpl`
+        parameter as the template.
+        
+        :param tmpl:
+            HTML template to use for rendering each item.
+        :param **kwargs:
+            Named paremeters used to generate the HTML attributes of each item.
+            It follows the same rules as `Field._get_html_attrs`
+        
+        """
+        if _items is None:
+            _items = self.get_items()
+        kwargs['type'] = 'radio'
+        kwargs['name'] = self.name
+        html = []
+        
+        curr_values = self.to_html()
+        if not isinstance(curr_values, list):
+            curr_values = [curr_values]
+
+        for value, label in _items:
+            item_attrs = kwargs.copy()
+            item_attrs['value'] = value
+            if value in curr_values or str(value) in curr_values:
                 item_attrs['checked'] = True
             html_attrs = self._get_html_attrs(item_attrs)
             html.append(tmpl % {'attrs': html_attrs, 'label': label})
         return Markup('\n'.join(html))
 
 
-class _Collection(_Text):
+class _Collection(_Text, MultiValue):
     """A field that takes an open number of values of the same kind.
     For example, a list of comma separated tags or email addresses.
 
@@ -823,65 +977,62 @@ class _Collection(_Text):
         When joining the values to render, it is used as-is. When splitting
         the user input, however, is tranformed first to a regexp
         when the spaces around the separator are ignored.
+
     :param filters:
         List of validators. If a value do not pass one of these it'll be
         filtered out from the final result.
+
+    :param validate:
+        An list of validators. This will evaluate the current `value` when
+        the method `validate` is called.
+
+    :param default:
+        Default value.
+
     :param prepare:
         An optional function that preprocess the value before loading.
+
     :param clean:
-        An optional function that takes a value and return a 'cleaned' version
-        of it. If a value raise an exception it'll be filtered out from the
-        final result.
-    :param validate:
-        An list of validators. This will evaluate each of the selected values
-        when the method `validate` is called.
+        An optional function that takes the value and return a 'cleaned'
+        version of it. If the value raise an exception, `None` must be
+        returned instead.
+
+    :param locale:
+        Default locale
+
+    :param tz:
+        Default timezone
 
-    Any other named parameter will be stored in `self.extra`.
     """
+    _type = 'text'
 
-    def __init__(self, sep=', ', filters=None, prepare=None, clean=None,
-            validate=None, **kwargs):
-        validate = validate or []
+    def __init__(self, sep=', ', **kwargs):
         self.sep = sep
         self.rxsep = r'\s*%s\s*' % self.sep.replace(' ', '')
-        filters = filters or []
-        self.filters = [f() if inspect.isclass(f) else f for f in filters]
-        self.prepare = prepare
-        self.clean = clean
-        super(_Collection, self).__init__(validate=validate, **kwargs)
-    
-    def to_html(self, locale=None, tz=None):
-        value = self.python_value or []
-        if self.prepare:
-            value = self.prepare(value)
-        return self.sep.join(value)
+        super(_Collection, self).__init__(**kwargs)
 
-    def to_python(self, locale=None, tz=None):
-        if not self._value:
+    def get_value(self):
+        values = self.value or self.obj_value
+        if not values:
             return []
-        return re.split(self.rxsep, self._value)
+        if isinstance(values, list):
+            return values
+        return re.split(self.rxsep, values)
+
+    def to_html(self, locale=None, tz=None):
+        if self.hide_value:
+            return u''
+        values = self.get_value()
+        values = self.prepare(values, locale, tz)
+        html = to_unicode(self.sep.join(values)) or u''
+        return html.strip()
 
-    def clean_value(self, python_value):
-        values = python_value or []
-        for f in self.filters:
-            values = filter(f, values)
-        if self.clean:
-            values_ = []
-            for value in values:
-                try:
-                    val = self.clean(value)
-                except Exception:
-                    continue
-                if val:
-                    values_.append(val)
-            values = values_
-        
-        return filter(lambda v: bool(v), values)
 
 
 #- Field factories
 #------------------------------------------------------------------------------#
 
+
 class Field(object):
     """A form field factory. All field factories must inherit from this class.
     """
@@ -931,9 +1082,6 @@ class Boolean(Field):
 class Select(Field):
     _class = _Select
 
-class SelectMulti(Field):
-    _class = _SelectMulti
-
 class Collection(Field):
     _class = _Collection
 
